﻿//-----------------------------------------------------------------------
// <copyright file="WikiServer.cs" company="BME MIT">
//     Simple Wiki server.
//     Created by Daniel Darvas, BME MIT FTSRG, 2012.
//     Website: https://www.inf.mit.bme.hu/edu/bsc/irf
// </copyright>
//-----------------------------------------------------------------------

using System.Collections.Generic;
using System.IO;
using System.Web;
using TinyServerLib;

namespace MicroWiki
{
    /// <summary>
    /// Wiki server engine class.
    /// Created by Daniel Darvas, BME MIT FTSRG, 2012.
    /// </summary>
    public class WikiServer : TinyServer
    {
        /// <summary>
        /// The heading of reading requests.
        /// </summary>
        private const string READURL = "/wiki/";

        /// <summary>
        /// The heading of editing requests.
        /// </summary>
        private const string EDITURL = "/edit/";

        /// <summary>
        /// Handles a received GET request.
        /// </summary>
        /// <param name="argument">The argument of the GET command.</param>
        /// <param name="parameters">The parameters of the GET command.</param>
        protected override void DoGet(string argument, Dictionary<string, string> parameters)
        {
            string path = argument.Substring(0, argument.IndexOf("HTTP/1"));

            if (path.StartsWith("/ "))
            {
                path = path.Replace("/ ", string.Format("{0}Home ", READURL));
            }

            if (path.StartsWith(READURL))
            {
                path = path.Replace(READURL, string.Empty);
                path = path.Replace(" ", string.Empty);

                this.GetWikiPage(path);
            }
            else if (path.StartsWith(EDITURL))
            {
                path = path.Replace(EDITURL, string.Empty);
                path = path.Replace(" ", string.Empty);

                this.GetWikiPageEditor(path);
            }
            else
            {
                this.SendError(404, "Not exists.");
            }
        }

        /// <summary>
        /// Sends a wiki page for the client.
        /// </summary>
        /// <param name="title">The title of the wiki page to be read.</param>
        private void GetWikiPage(string title)
        {
            string filePath = Path.Combine(this.WebRootPath, title + ".txt");

            this.SendOk();

            string content = string.Empty;
            string template = File.ReadAllText(this.GetTemplatePath("pageTemplate.html"));
            if (File.Exists(filePath))
            {
                content = File.ReadAllText(filePath);
            }
            else
            {
                content = string.Format("This page does not exists. <a href='/edit/{0}'>Create it!</a>", title);
            }

            content = content.Replace("\n", "<br>");

            content = WikiHelper.ReplaceLinks(content);
            content = WikiHelper.ReplaceFormatSigns(content);

            template = template.Replace("%Title%", title);
            template = template.Replace("%Content%", content);

            this.SendString(template);
        }

        /// <summary>
        /// Sends a wiki page editor for the client.
        /// </summary>
        /// <param name="title">The title of the wiki page to be edited.</param>
        private void GetWikiPageEditor(string title)
        {
            string filePath = Path.Combine(this.WebRootPath, title + ".txt");

            this.SendOk();
            string template = File.ReadAllText(this.GetTemplatePath("editTemplate.html"));
            if (File.Exists(filePath))
            {
                string content = File.ReadAllText(filePath);
                template = template.Replace("%Content%", content);
            }
            else
            {
                template = template.Replace("%Content%", string.Empty);
            }

            template = template.Replace("%Title%", title);
            this.SendString(template);
        }

        /// <summary>
        /// Handles a received POST request.
        /// </summary>
        /// <param name="argument">The argument of the POST command.</param>
        /// <param name="parameters">The parameters of the POST command.</param>
        /// <param name="content">The content of the POST command.</param>
        protected override void DoPost(string argument, Dictionary<string, string> parameters, string content)
        {
            string path = argument.Substring(0, argument.IndexOf("HTTP"));
            path = path.Replace(READURL, string.Empty);
            path = path.Replace(" ", string.Empty);

            string filePath = Path.Combine(this.WebRootPath, path + ".txt");
            string fileContent = content.Substring(content.IndexOf('=') + 1);
            fileContent = HttpUtility.UrlDecode(fileContent);

            Directory.CreateDirectory(new FileInfo(filePath).DirectoryName);
            File.WriteAllText(filePath, fileContent);

            this.DoGet(argument, parameters);
        }
    }
}
